﻿// Script for calling enfuse.exe
// download enfuse from http://enblend.sourceforge.net/
//
// This is a script for Directory Opus.
// See http://www.gpsoft.com.au/DScripts/redirect.asp?page=scripts for development information.
// 
// Called by Directory Opus to initialize the script

var resourceMtnName = "enfuse";
var resourceMtnKey = "enfuse_ExePath";

var doFsu = DOpus.FSUtil;

function OnInit(initData)
{
	initData.name = "Enfuse";
	initData.desc = "Enfuse selected images";
	initData.copyright = "wowbagger";
	initData.version = "1.5";
	initData.default_enable = false;
  initData.config_desc = DOpus.NewMap();
  
  // variable for enabling debug, so as to enable debug via a button
  initData.config.DebugEnableVar = "$glob:debug"
  initData.config.DEBUG = true
  
  initData.config.EnfuseExePath = "C:\\bin\\enfuse\\bin\\enfuse.exe";
  initData.config.HideRuntime = false;
  
  initData.config_desc("ExposureStackParams") = "Blends a stack of images with differing exposures (default option):";
  initData.config.ExposureStackParams = "";
    
  initData.config_desc("FocusStackParams") = "Blends a stack of images with differing focus to form an extended depth-of-field image:";
  initData.config.FocusStackParams = "--exposure-weight=0.001 --saturation-weight=0.001 --contrast-weight=1 --hard-mask";
  
  initData.config_desc("CustomDefaultParams") = "Default value for the customise dialog";
  initData.config.CustomDefaultParams = "--exposure-weight=1 --saturation-weight=0.2 --contrast-weight=0 --exposure-mu=0.5 --soft-mask";
  
  initData.config_desc("UserDefaultParams") = "User pattern";
  initData.config.UserDefaultParams = "--exposure-weight=0.001 --saturation-weight=1 --contrast-weight=0 --exposure-mu=.9 --hard-mask";
  
	var cmd = initData.AddCommand();
	cmd.name = "EnfuseSelectedFiles";
	cmd.method = "OnEnfuseSelectedFiles";
	cmd.desc = "Enfuses selected files ";
	cmd.label = "Enfuse Selected Files";
	cmd.template = "FocusStack/O,UserDefined/O,CustomPrams/KO,DialogDefined/O";
  
  var cmd = initData.AddCommand();
	cmd.name = "EnfuseCommandLineHelp";
	cmd.method = "onHelp";
	cmd.desc = "Get the Enfuses.exe command line options";
	cmd.label = "Enfuse Selected Files";
	cmd.template = "FocusStack/O,UserDefined/O,CustomPrams/KO,DialogDefined/O";
}

function onHelp()
{
	Config = "*** Enfuse Command Line Parameters ***\n";
	Config += "\n";
	Config += "--exposure-weight=WEIGHT\n";
	Config += "Sets the relative WEIGHT of the well-exposedness criterion. Increasing this weight relative to the others will make well-exposed pixels contribute more to the final output.\n";
  Config += "Default: 1.0. Valid range: 0 ≤ WEIGHT ≤ 1 .\n\n";
	Config += "--saturation-weight=WEIGHT\n";
	Config += "Sets the relative WEIGHT of high-saturation pixels. Increasing this weight makes pixels with high saturation contribute more to the final output.\n";
	Config += " Default: 0.2. Valid range: 0 ≤ WEIGHT ≤ 1 . \n\n";
	Config += "--contrast-weight=WEIGHT\n";
	Config += "Sets the relative WEIGHT of high local-contrast pixels.\n";
	Config += "Default: 0.0. Valid range: 0 ≤ WEIGHT ≤ 1 .\n\n";
	Config += "--exposure-mu=MEAN\n";
	Config += "Set the MEAN (this is, the center) of the Gaussian exposure weight curve.\n";
	Config += "Default: 0.5. Valid range: 0 ≤ MEAN ≤ 1 . \n\n";
	Config += "soft-mask\n";
	Config += "Defines the details of log information the script outputs to the dopus other log.\n";
	Config += "Default: 0\n";
	Config += "More Details http://enblend.sourceforge.net/enfuse.doc/enfuse_4.0.0.xhtml/Invocation.xhtml#Invocation\n";
  return Config;
}

function IsDebugEnabled()
{
  return Script.config.DEBUG || doCmd.IsSet(Script.config.DebugEnableVar);
}

function LogMessage(message)
{
  if (IsDebugEnabled()) { DOpus.OutputString(message)};
}
  
// Implement the CustomCommand command
function OnEnfuseSelectedFiles(funcData)
{
  //var exposureStack = funcData.func.args.got_arg.ExposureStack ? funcData.func.args.ExposureStack : false; 
  var focusStack = funcData.func.args.got_arg.FocusStack ? funcData.func.args.FocusStack : false; 
  var userDefined = funcData.func.args.got_arg.UserDefined ? funcData.func.args.UserDefined : false; 
  var customPrams = funcData.func.args.got_arg.CustomPrams ? funcData.func.args.CustomPrams : null; 
  var dialogDefined = funcData.func.args.got_arg.DialogDefined ? funcData.func.args.DialogDefined : false; 
  
  var params;
  if(dialogDefined)
  {
    var dlg = DOpus.Dlg;
    dlg.window = funcData.func.sourcetab;
    dlg.template = "dialogEnfuse";
    dlg.detach = true;
    
    dlg.Show();
  
    dlg.Control("comboPresets").AddItem("ExposureStackParams");
    dlg.Control("comboPresets").AddItem("FocusStackParams");
    dlg.Control("comboPresets").AddItem("CustomDefaultParams");
    dlg.Control("comboPresets").AddItem("UserDefaultParams");
  
    var closeDialog;
    while (!closeDialog) {
      var msg = dlg.GetMsg();
      if (!msg.result) break;
      
      LogMessage("Msg: " + msg.control + " value: " + msg.value + " event:" + msg.event);

      switch (msg.control) {
        case "comboPresets" :
           PresetSelected(dlg, msg);
           break;
        case "buttonOK" :
           params = DialogClosed(dlg, msg);
           closeDialog = true;
           break;
      } 
    }

    if(!(params))
    {
      LogMessage("Dialog canceled");
      return;
    }

    LogMessage("Return code = " + dlg.result);
    LogMessage("Return value = " + params);
  }
  else if(customPrams)
  {
    params = customPrams;
  }
  else if (userDefined)
  {
    params = Script.config.UserDefaultParams;
  }
  else if (focusStack)
  {
    params = Script.config.FocusStackParams;
  }
  else
  {
    params = Script.config.ExposureStackParams; //Default option
  }
    
  var doCmd = DOpus.NewCommand;
	var objEnum = new Enumerator( funcData.func.sourcetab.selected_files );
  
  var targetPath;
  var fileNames = "";
    
  while (!objEnum.atEnd())
  {
    var file = objEnum.item();
    fileNames += " \"" + file + "\"";
    if(!targetPath)
    {
      targetPath = file + ".enfuse" + dateTimeStamp(new Date()) +".tif"
    }
    objEnum.moveNext();  
  }
   
  var enfuseExePath = Script.config.EnfuseExePath;

  if(!doFsu.Exists(enfuseExePath))
  {
    LogMessage("Can't find the enfuse exe, Path: " + enfuseExePath);
    return;
  }
  
  var cmdParams = "-o \"" + targetPath + "\" " + params + fileNames + " ";
  
  var cmd = "@sync:\"" + enfuseExePath + "\" " + cmdParams;
  
  LogMessage("enfusing files target \"" + targetPath + "\" using \"" + enfuseExePath + "\"\ncmd:" + cmd);
  if(!IsDebugEnabled() && Script.config.HideRuntime)
  {
    doCmd.SetModifier("runmode", "hide");
  }

  var result = doCmd.RunCommand(cmd);
}

function PresetSelected(dlg, msg) {
  LogMessage("PresetSelected: " + msg.control +":"+ msg.event);
 
  if(msg.control != "comboPresets" || msg.event != "selchange")
    return;
    
  var dialogParams;
  switch (msg.value) {
    case "ExposureStackParams" :
      dialogParams = Script.config.ExposureStackParams;
      break;
    case "FocusStackParams" :
      dialogParams = Script.config.FocusStackParams;
      break;
    case "CustomDefaultParams" :
      dialogParams = Script.config.CustomDefaultParams;
      break;
    case "UserDefaultParams" :
      dialogParams = Script.config.UserDefaultParams;
      break;
  } 

  if(!(dialogParams))
  {
    dialogParams= "--exposure-weight=--saturation-weight=--contrast-weight=--exposure-mu=--soft-mask"
  }

  if(dialogParams)
  {
    var splitParams = dialogParams.split("--");
    for (var i = 0; i < splitParams.length; i++) {
      DOpus.Output(splitParams[i]);
      var splitParam = splitParams[i].split("=");
      switch (splitParam[0]) {
        case "exposure-weight" :
          dlg.Control("exposure").value = splitParam[1];
          break;
        case "saturation-weight" :
          dlg.Control("saturation").value = splitParam[1];
          break;
        case "contrast-weight" :
          dlg.Control("contrast").value = splitParam[1];
          break;
        case "exposure-mu" :
          dlg.Control("gaussion").value = splitParam[1];
          break;
        case "soft-mask" :
          dlg.Control("mask").value = 0;
          break;
        case "hard-mask" :
          dlg.Control("mask").value = 1;
          break;
      } 
    }
  }
}

function Trim(s) { return s.replace(/(^\s+)|(\s+$)/g, "") } 

function BuildCommand(command, value) 
{
  if(value)
  {
    return " --" + command + "=" + Trim(value)   
  }
  return "";
} 


function DialogClosed(dlg, msg) {
  var result = BuildCommand("exposure-weight", dlg.Control("exposure").value)
             + BuildCommand("saturation-weight", dlg.Control("saturation").value)
             + BuildCommand("contrast-weight", dlg.Control("contrast").value)
             + BuildCommand("exposure-mu", dlg.Control("gaussion").value)
             + ((dlg.Control("mask").value == 1) ? " --hard-mask" : " --soft-mask");
  LogMessage("selected: " + result + " || " + dlg.Control("mask").value);
  return result;
}

function dateTimeStamp(date) {
  var month = date.getMonth() + 1;
  var day = date.getDate();
  var hour = date.getHours();
  var minuites = date.getMinutes(); 
  var seconds = date.getSeconds(); 
  var dateOfString = date.getFullYear()
    + (("" + month).length < 2 ? "0" : "") + month
    + (("" + day).length < 2 ? "0" : "") + day + "T"
    + (("" + hour).length < 2 ? "0" : "") + hour
    + (("" + minuites).length < 2 ? "0" : "") + minuites
    + (("" + seconds).length < 2 ? "0" : "") + seconds;
  return dateOfString;
}

// Called to display an About dialog for this script
function OnAboutScript(aboutData)
{
	var dlg = DOpus.Dlg();
	dlg.window = aboutData.window;
	dlg.message = "Script for calling enfuse.exe\r\ndownload enfuse from http://enblend.sourceforge.net \r\n";
	dlg.title = "About enfuse";
	dlg.buttons = "OK";
	dlg.icon = "info";
	dlg.Show();
}

==SCRIPT RESOURCES
<resources>
  <resource name="dialogEnfuse" type="dialog">
    <dialog fontsize="8" height="174" lang="english" title="Enfuse Custom execution" width="210">
      <control close="0" height="14" name="buttonCancel" title="Cancel" type="button" width="50" x="156" y="156" />
      <control default="yes" height="14" name="buttonOK" title="OK" type="button" width="50" x="96" y="156" />

      <control halign="left" height="8" name="static6" title="Presets" type="static" width="30" x="6" y="6" />
      <control height="40" name="comboPresets" type="combo" width="198" x="6" y="18" />
  
      <control height="102" name="group1" title="Group" type="group" width="198" x="6" y="36" />
      <control halign="left" height="12" name="staticSaturation" title="Saturation Weight" type="static" width="66" x="12" y="66" />
      <control halign="left" height="12" name="staticContrast" title="Contrast Weight" type="static" width="66" x="12" y="84" />
      <control halign="left" height="12" name="staticGaussion" title="Exposure MU" type="static" width="66" x="12" y="102" />
      <control halign="left" height="12" name="staicMask" title="Mask" type="static" width="66" x="12" y="120" />
      <control halign="left" height="12" name="staticExposure" title="Exposure Weight" type="static" width="66" x="12" y="48" />
      <control halign="left" height="12" name="exposure" tip="(0 &lt;= WEIGHT &lt;= 1); default: 1" type="edit" width="120" x="78" y="48" />
      <control halign="left" height="12" name="saturation" tip="(0 &lt;= WEIGHT &lt;= 1); default: 0.2" type="edit" width="120" x="78" y="66" />
      <control halign="left" height="12" name="contrast" tip="(0 &lt;= WEIGHT &lt;= 1); default: 0" type="edit" width="120" x="78" y="84" />
      <control halign="left" height="12" name="gaussion" tip="(0 &lt;= MEAN &lt;= 1); default: 0.5" type="edit" width="120" x="78" y="102" />
      <control height="40" name="mask" type="combo" width="64" x="78" y="120">
        <contents>
          <item text="Soft mask" />
          <item text="Hard mask" />
        </contents>
      </control>
    </dialog>
  </resource>
</resources>
